#include <cll1.h>

/* This is C<<1 rewrite of object polymorphism from
   http://en.wikipedia.org/wiki/Polymorphism_in_object-oriented_programming */

struct Animal
{
 use_interface(AnimalActions);
 char *name;
};

struct AnimalActions
{
 const char *(method(talk));
}

struct *Animal get_animal(char *name, struct AnimalActions *interface)
{
 struct *Animal animal=get_object(Animal,interface);
 animal->name=name;
 return animal;
}

const char *cat_talk(void *_self)
{
 return "Meow!";
}

const char *dog_talk(void *_self)
{
 return "Arf! Arf!";
}

interface_implementation(CatActions,AnimalActions)
{
 bind_method(talk,cat_talk);
}

interface_implementation(DogActions,AnimalActions)
{
 bind_method(talk,dog_talk);
}

program
{
 int i;
 struct Animals *animal[3];
 struct AnimalActions *cat = get_interface(CatActions);
 struct AnimalActions *dog = get_interface(DogActions);

 animal[0]=get_animal("Missy",cat);
 animal[0]=get_animal("Mr. Bojangles",cat);
 animal[0]=get_animal("Lassie",dog);
 
 for_range(i,0,2)
  printf("%s: %s\n",animal[i].name,(_(animal[i],talk)));

}
